@@ -23,39 +23,22 @@
 
 #include "gkd-login.h"
 
+#include "daemon/gkd-pkcs11.h"
+
+#include "egg/egg-error.h"
 #include "egg/egg-secure-memory.h"
 
-#include "pkcs11/gkd-pkcs11.h"
 #include "pkcs11/pkcs11i.h"
+#include "pkcs11/wrap-layer/gkm-wrap-layer.h"
 
 #include <glib/gi18n.h>
 
 #include <string.h>
 
-static gint unlock_failures = 0;
-
-static void
-note_that_unlock_failed (void)
-{
-	g_atomic_int_inc (&unlock_failures);
-}
-
-static void
-note_that_unlock_succeeded (void)
-{
-	g_atomic_int_set (&unlock_failures, 0);
-}
-
-gboolean
-gkd_login_did_unlock_fail (void)
-{
-	return g_atomic_int_get (&unlock_failures) ? TRUE : FALSE;
-}
-
 static GP11Module*
 module_instance (void)
 {
-	GP11Module *module = gp11_module_new (gkd_pkcs11_get_base_functions ());
+	GP11Module *module = gp11_module_new (gkd_pkcs11_get_functions ());
 	gp11_module_set_pool_sessions (module, FALSE);
 	gp11_module_set_auto_authenticate (module, FALSE);
 	g_return_val_if_fail (module, NULL);
@@ -76,7 +59,9 @@ open_and_login_session (GP11Slot *slot, CK_USER_TYPE user_type, GError **error)
 	session = gp11_slot_open_session (slot, CKF_RW_SESSION, error);
 	if (session != NULL) {
 		if (!gp11_session_login (session, user_type, NULL, 0, error)) {
-			if ((*error)->code != CKR_USER_ALREADY_LOGGED_IN) {
+			if (g_error_matches (*error, GP11_ERROR, CKR_USER_ALREADY_LOGGED_IN)) {
+				g_clear_error (error);
+			} else {
 				g_object_unref (session);
 				session = NULL;
 			}
@@ -117,8 +102,8 @@ lookup_login_session (GP11Module *module)
 	g_return_val_if_fail (slot, NULL);
 
 	session = open_and_login_session (slot, CKU_USER, &error);
-	if (session == NULL) {
-		g_warning ("couldn't open pkcs11 session for login: %s", error->message);
+	if (error) {
+		g_warning ("couldn't open pkcs11 session for login: %s", egg_error_message (error));
 		g_clear_error (&error);
 	}
 
@@ -144,7 +129,7 @@ lookup_login_keyring (GP11Session *session)
 	                                     GP11_INVALID);
 
 	if (error) {
-		g_warning ("couldn't search for login keyring: %s", error->message);
+		g_warning ("couldn't search for login keyring: %s", egg_error_message (error));
 		g_clear_error (&error);
 		return NULL;
 	}
@@ -238,23 +223,23 @@ unlock_or_create_login (GP11Module *module, const gchar *master)
 
 	/* Failure, bad password? */
 	if (cred == NULL) {
-		if (login && error->code == CKR_PIN_INCORRECT)
-			note_that_unlock_failed ();
+		if (login && g_error_matches (error, GP11_ERROR, CKR_PIN_INCORRECT))
+			gkm_wrap_layer_hint_login_unlock_failure ();
 		else
-			g_warning ("couldn't create login credential: %s", error->message);
+			g_warning ("couldn't create login credential: %s", egg_error_message (error));
 		g_clear_error (&error);
 
 	/* Non login keyring, create it */
 	} else if (!login) {
 		login = create_login_keyring (session, cred, &error);
-		if (login == NULL) {
-			g_warning ("couldn't create login keyring: %s", error->message);
+		if (login == NULL && error) {
+			g_warning ("couldn't create login keyring: %s", egg_error_message (error));
 			g_clear_error (&error);
 		}
 
 	/* The unlock succeeded yay */
 	} else {
-		note_that_unlock_succeeded ();
+		gkm_wrap_layer_hint_login_unlock_success ();
 	}
 
 	if (cred)
@@ -287,14 +272,10 @@ init_pin_for_uninitialized_slots (GP11Module *module, const gchar *master)
 		if (initialize) {
 			session = open_and_login_session (l->data, CKU_SO, NULL);
 			if (session != NULL) {
-				if (gp11_session_init_pin (session, (const guchar*)master, strlen (master), &error)) {
-					gkd_login_attach_secret (info->label, master,
-					                         "manufacturer", info->manufacturer_id,
-					                         "serial-number", info->serial_number,
-					                         NULL);
-				} else {
-					if (error->code != CKR_FUNCTION_NOT_SUPPORTED)
-						g_warning ("couldn't initialize slot with master password: %s", error->message);
+				if (!gp11_session_init_pin (session, (const guchar*)master, strlen (master), &error)) {
+					if (!g_error_matches (error, GP11_ERROR, CKR_FUNCTION_NOT_SUPPORTED))
+						g_warning ("couldn't initialize slot with master password: %s",
+						           egg_error_message (error));
 					g_clear_error (&error);
 				}
 				g_object_unref (session);
@@ -348,19 +329,21 @@ change_or_create_login (GP11Module *module, const gchar *original, const gchar *
 	/* Create the new credential we'll be changing to */
 	mcred = create_credential (session, NULL, master, &error);
 	if (mcred == NULL) {
-		g_warning ("couldn't create new login credential: %s", error->message);
+		g_warning ("couldn't create new login credential: %s", egg_error_message (error));
 		g_clear_error (&error);
 
 	/* Create original credentials */
 	} else if (login) {
 		ocred = create_credential (session, login, original, &error);
 		if (ocred == NULL) {
-			if (error->code == CKR_PIN_INCORRECT) {
+			if (g_error_matches (error, GP11_ERROR, CKR_PIN_INCORRECT)) {
 				g_message ("couldn't change login master password, "
-				           "original password was wrong: %s", error->message);
-				note_that_unlock_failed ();
+				           "original password was wrong: %s",
+				           egg_error_message (error));
+				gkm_wrap_layer_hint_login_unlock_failure ();
 			} else {
-				g_warning ("couldn't create original login credential: %s", error->message);
+				g_warning ("couldn't create original login credential: %s",
+				           egg_error_message (error));
 			}
 			g_clear_error (&error);
 		}
@@ -370,7 +353,7 @@ change_or_create_login (GP11Module *module, const gchar *original, const gchar *
 	if (!login && mcred) {
 		login = create_login_keyring (session, mcred, &error);
 		if (login == NULL) {
-			g_warning ("couldn't create login keyring: %s", error->message);
+			g_warning ("couldn't create login keyring: %s", egg_error_message (error));
 			g_clear_error (&error);
 		} else {
 			success = TRUE;
@@ -381,7 +364,7 @@ change_or_create_login (GP11Module *module, const gchar *original, const gchar *
 		if (!gp11_object_set (login, &error,
 		                      CKA_G_CREDENTIAL, GP11_ULONG, gp11_object_get_handle (mcred),
 		                      GP11_INVALID)) {
-			g_warning ("couldn't change login master password: %s", error->message);
+			g_warning ("couldn't change login master password: %s", egg_error_message (error));
 			g_clear_error (&error);
 		} else {
 			success = TRUE;
@@ -425,15 +408,12 @@ set_pin_for_any_slots (GP11Module *module, const gchar *original, const gchar *m
 		if (initialize) {
 			session = open_and_login_session (l->data, CKU_USER, NULL);
 			if (session != NULL) {
-				if (gp11_session_set_pin (session, (const guchar*)original, strlen (original),
-				                          (const guchar*)master, strlen (master), &error)) {
-					gkd_login_attach_secret (info->label, master,
-					                         "manufacturer", info->manufacturer_id,
-					                         "serial-number", info->serial_number,
-					                         NULL);
-				} else {
-					if (error->code != CKR_PIN_INCORRECT && error->code != CKR_FUNCTION_NOT_SUPPORTED)
-						g_warning ("couldn't change slot master password: %s", error->message);
+				if (!gp11_session_set_pin (session, (const guchar*)original, strlen (original),
+				                           (const guchar*)master, strlen (master), &error)) {
+					if (!g_error_matches (error, GP11_ERROR, CKR_PIN_INCORRECT) &&
+					    !g_error_matches (error, GP11_ERROR, CKR_FUNCTION_NOT_SUPPORTED))
+						g_warning ("couldn't change slot master password: %s",
+						           egg_error_message (error));
 					g_clear_error (&error);
 				}
 				g_object_unref (session);
@@ -467,249 +447,3 @@ gkd_login_change_lock (const gchar *original, const gchar *master)
 	g_object_unref (module);
 	return result;
 }
-
-gboolean
-gkd_login_is_usable (void)
-{
-	GP11Module *module;
-	GP11Session *session;
-	GP11Object *login;
-	gboolean usable = FALSE;
-	gpointer data;
-	gsize n_data;
-
-	module = module_instance ();
-	if (!module)
-		return FALSE;
-
-	session = lookup_login_session (module);
-	if (session) {
-		login = lookup_login_keyring (session);
-		if (login) {
-			data = gp11_object_get_data (login, CKA_G_LOCKED, &n_data, NULL);
-			usable = (data && n_data == sizeof (CK_BBOOL) && !*((CK_BBOOL*)data));
-			g_free (data);
-			g_object_unref (login);
-		}
-		g_object_unref (session);
-	}
-
-	g_object_unref (module);
-	return usable;
-}
-
-static void
-string_attribute_list_va (va_list args, const gchar *name, GP11Attribute *attr)
-{
-	GString *fields = g_string_sized_new(128);
-	gsize length;
-
-	while (name != NULL) {
-		g_string_append (fields, name);
-		g_string_append_c (fields, '\0');
-		g_string_append (fields, va_arg (args, const gchar*));
-		g_string_append_c (fields, '\0');
-		name = va_arg (args, const gchar*);
-	}
-
-	length = fields->len;
-	gp11_attribute_init (attr, CKA_G_FIELDS, g_string_free (fields, FALSE), length);
-}
-
-static GP11Object*
-find_login_keyring_item (GP11Session *session, GP11Attribute *fields)
-{
-	GP11Object *search;
-	GP11Object *item = NULL;
-	GList *objects;
-	GError *error = NULL;
-	gpointer data;
-	gsize n_data;
-
-	g_return_val_if_fail (GP11_IS_SESSION (session), FALSE);
-
-	/* Create a search object */
-	search = gp11_session_create_object (session, &error,
-	                                     CKA_CLASS, GP11_ULONG, CKO_G_SEARCH,
-	                                     CKA_G_COLLECTION, (gsize)5, "login",
-	                                     CKA_TOKEN, GP11_BOOLEAN, FALSE,
-	                                     CKA_G_FIELDS, fields->length, fields->value,
-	                                     GP11_INVALID);
-
-	if (!search) {
-		g_warning ("couldn't create search for login keyring: %s", error->message);
-		g_clear_error (&error);
-		return NULL;
-	}
-
-	/* Get the data from the search */
-	gp11_object_set_session (search, session);
-	data = gp11_object_get_data (search, CKA_G_MATCHED, &n_data, &error);
-	gp11_object_destroy (search, NULL);
-	g_object_unref (search);
-
-	if (data == NULL) {
-		g_warning ("couldn't read search in login keyring: %s", error->message);
-		g_clear_error (&error);
-		return NULL;
-	}
-
-	n_data /= sizeof (CK_OBJECT_HANDLE);
-	objects = gp11_objects_from_handle_array (gp11_session_get_slot (session), data,
-	                                          MIN (sizeof (CK_OBJECT_HANDLE), n_data));
-	g_free (data);
-
-	if (objects) {
-		item = g_object_ref (objects->data);
-		gp11_object_set_session (item, session);
-	}
-
-	gp11_list_unref_free (objects);
-	return item;
-}
-
-void
-gkd_login_attach_secret (const gchar *label, const gchar *secret,
-                         const gchar *first, ...)
-{
-	GError *error = NULL;
-	GP11Attribute fields;
-	GP11Session *session;
-	GP11Module *module;
-	gchar *display_name;
-	GP11Object* item;
-	va_list va;
-
-	if (label == NULL)
-		label = _("Unnamed");
-	if (secret == NULL)
-		secret = "";
-
-	module = module_instance ();
-	session = lookup_login_session (module);
-
-	va_start(va, first);
-	gp11_attribute_init_empty (&fields, CKA_G_FIELDS);
-	string_attribute_list_va (va, first, &fields);
-	va_end(va);
-
-	display_name = g_strdup_printf (_("Unlock password for: %s"), label);
-
-	item = find_login_keyring_item (session, &fields);
-	if (item) {
-		gp11_object_set (item, &error,
-		                 CKA_LABEL, strlen (display_name), display_name,
-		                 CKA_VALUE, strlen (secret), secret,
-		                 GP11_INVALID);
-	} else {
-		item = gp11_session_create_object (session, &error,
-		                                   CKA_TOKEN, GP11_BOOLEAN, TRUE,
-		                                   CKA_CLASS, GP11_ULONG, CKO_SECRET_KEY,
-		                                   CKA_LABEL, strlen (display_name), display_name,
-		                                   CKA_VALUE, strlen (secret), secret,
-		                                   CKA_G_COLLECTION, (gsize)5, "login",
-		                                   CKA_G_FIELDS, fields.length, fields.value,
-		                                   GP11_INVALID);
-	}
-
-	if (error != NULL) {
-		g_warning ("couldn't store secret in login keyring: %s", error->message);
-		g_clear_error (&error);
-	}
-
-	if (item)
-		g_object_unref (item);
-	g_free (display_name);
-	gp11_attribute_clear (&fields);
-	g_object_unref (session);
-	g_object_unref (module);
-}
-
-gchar*
-gkd_login_lookup_secret (const gchar *first, ...)
-{
-	GP11Attribute fields;
-	GP11Session *session;
-	GP11Module *module;
-	GP11Object* item;
-	gpointer data = NULL;
-	gsize n_data;
-	va_list va;
-
-	module = module_instance ();
-	session = lookup_login_session (module);
-
-	va_start(va, first);
-	gp11_attribute_init_empty (&fields, CKA_G_FIELDS);
-	string_attribute_list_va (va, first, &fields);
-	va_end(va);
-
-	item = find_login_keyring_item (session, &fields);
-	if (item != NULL) {
-		data = gp11_object_get_data_full (item, CKA_VALUE, egg_secure_realloc, NULL, &n_data, NULL);
-		if (data && !g_utf8_validate (data, n_data, NULL)) {
-			g_warning ("expected string, but found binary secret in login keyring");
-			egg_secure_clear (data, n_data);
-			egg_secure_free (data);
-			data = NULL;
-		}
-		g_object_unref (item);
-	}
-
-	g_object_unref (session);
-	g_object_unref (module);
-
-	/* Memory returned from gp11_object_get_data is null terminated */
-	return data;
-}
-
-void
-gkd_login_remove_secret (const gchar *first, ...)
-{
-	GError *error = NULL;
-	GP11Attribute fields;
-	GP11Session *session;
-	GP11Module *module;
-	GP11Object* item;
-	va_list va;
-
-	module = module_instance ();
-	session = lookup_login_session (module);
-
-	va_start(va, first);
-	gp11_attribute_init_empty (&fields, CKA_G_FIELDS);
-	string_attribute_list_va (va, first, &fields);
-	va_end(va);
-
-	item = find_login_keyring_item (session, &fields);
-	if (item != NULL) {
-		if (!gp11_object_destroy (item, &error)) {
-			if (error->code != CKR_OBJECT_HANDLE_INVALID)
-				g_warning ("couldn't remove stored secret from login keyring: %s", error->message);
-			g_clear_error (&error);
-		}
-		g_object_unref (item);
-	}
-
-	g_object_unref (session);
-	g_object_unref (module);
-}
-
-GP11Attributes*
-gkd_login_attributes_for_secret (const gchar *first, ...)
-{
-	GP11Attributes *attrs;
-	GP11Attribute *fields;
-	va_list va;
-
-	attrs = gp11_attributes_newv (CKA_CLASS, GP11_ULONG, CKO_SECRET_KEY,
-	                              CKA_G_COLLECTION, (gsize)5, "login",
-	                              GP11_INVALID);
-
-	va_start(va, first);
-	fields = gp11_attributes_add_empty (attrs, CKA_G_FIELDS);
-	string_attribute_list_va (va, first, fields);
-	va_end(va);
-
-	return attrs;
-}
