@@ -24,6 +24,7 @@
 #include "config.h"
 
 #include "egg-asn1.h"
+#include "egg-oid.h"
 
 #include <libtasn1.h>
 
@@ -221,6 +222,7 @@ egg_asn1_read_value (ASN1_TYPE asn, const gchar *part, gsize *len, EggAllocator
 	
 	g_return_val_if_fail (asn != NULL, NULL);
 	g_return_val_if_fail (part != NULL, NULL);
+	g_return_val_if_fail (len != NULL, NULL);
 	
 	if (allocator == NULL)
 		allocator = (EggAllocator)g_realloc;
@@ -240,7 +242,7 @@ egg_asn1_read_value (ASN1_TYPE asn, const gchar *part, gsize *len, EggAllocator
 	if (res != ASN1_SUCCESS) {
 		(allocator) (buf, 0);
 		buf = NULL;
-	} else if (len) {
+	} else {
 		*len = l;
 	}
 	
@@ -261,20 +263,42 @@ egg_asn1_write_value (ASN1_TYPE asn, const gchar *part,
 	return res == ASN1_SUCCESS;
 }
 
+static gboolean
+ascii_length_equals (const gchar *str, gconstpointer data, gsize n_data)
+{
+	g_assert (str);
+	if (!data)
+		return FALSE;
+	if (strlen (str) != n_data)
+		return FALSE;
+	return strncmp (str, data, n_data) == 0;
+}
+
+static gboolean
+ascii_length_case_equals (const gchar *str, gconstpointer data, gsize n_data)
+{
+	g_assert (str);
+	if (!data)
+		return FALSE;
+	if (strlen (str) != n_data)
+		return FALSE;
+	return g_ascii_strncasecmp (str, data, n_data) == 0;
+}
+
 gboolean
 egg_asn1_read_boolean (ASN1_TYPE asn, const gchar *part, gboolean *val)
 {
 	gchar buffer[32];
-	int n_buffer = sizeof (buffer) - 1;
+	int n_buffer = sizeof (buffer);
 	int res;
 	
 	memset (buffer, 0, sizeof (buffer));
 	
 	res = asn1_read_value (asn, part, buffer, &n_buffer);
-	if (res != ASN1_SUCCESS)
+	if (res != ASN1_SUCCESS || !n_buffer)
 		return FALSE;
 		
-	if (g_ascii_strcasecmp (buffer, "TRUE") == 0)
+	if (ascii_length_case_equals ("TRUE", buffer, n_buffer - 1))
 		*val = TRUE;
 	else
 		*val = FALSE;
@@ -331,15 +355,20 @@ egg_asn1_read_oid (ASN1_TYPE asn, const gchar *part)
 {
 	GQuark quark;
 	guchar *buf;
+	gpointer end;
 	gsize n_buf;
 	
 	buf = egg_asn1_read_value (asn, part, &n_buf, NULL);
-	if (!buf)
+	if (!buf || !n_buf)
 		return 0;
-		
+
+	/* Make sure the string is actually that long */
+	end = memchr (buf, 0, n_buf - 1);
+	if (end != NULL)
+		return 0;
+
 	quark = g_quark_from_string ((gchar*)buf);
 	g_free (buf);
-	
 	return quark;
 }
 
@@ -442,9 +471,9 @@ time_t timegm(struct tm *t)
 #endif //NOT_HAVE_TIMEGM
 
 static gboolean
-parse_utc_time (const gchar *time, struct tm* when, gint *offset)
+parse_utc_time (const gchar *time, gsize n_time,
+                struct tm* when, gint *offset)
 {
-	guint n_time;
 	const char *p, *e;
 	int year;
 
@@ -452,7 +481,8 @@ parse_utc_time (const gchar *time, struct tm* when, gint *offset)
 	g_assert (time);
 	g_assert (offset);
 	
-	n_time = strlen (time);
+	if (n_time != strlen (time))
+		return FALSE;
 	
 	/* YYMMDDhhmmss.ffff Z | +0000 */
 	if (n_time < 6 || n_time >= 28) 
@@ -571,31 +601,35 @@ when_to_time (struct tm *when, gint offset)
 	return result;
 }
 
-time_t
-egg_asn1_time_parse_utc (const gchar *time)
+glong
+egg_asn1_time_parse_utc (const gchar *time, gssize n_time)
 {
 	struct tm when;
 	gint offset;
 	
 	g_return_val_if_fail (time, -1);
+
+	if (n_time == -1)
+		n_time = strlen (time);
 	
-	if (!parse_utc_time (time, &when, &offset))
+	if (!parse_utc_time (time, n_time, &when, &offset))
 		return -1;
 	
 	return when_to_time (&when, offset);
 }
 
 static gboolean
-parse_general_time (const gchar *time, struct tm* when, gint *offset)
+parse_general_time (const gchar *time, gsize n_time,
+                    struct tm* when, gint *offset)
 {
 	const char *p, *e;
-	guint n_time;
 
 	g_assert (time);
 	g_assert (when);
 	g_assert (offset);
-	
-	n_time = strlen (time);
+
+	if (strlen (time) != n_time)
+		return FALSE;
 	
 	/* YYYYMMDDhhmmss.ffff Z | +0000 */
 	if (n_time < 8 || n_time >= 30) 
@@ -689,15 +723,18 @@ parse_general_time (const gchar *time, struct tm* when, gint *offset)
 	return TRUE;
 }
 
-time_t
-egg_asn1_time_parse_general (const gchar *time)
+glong
+egg_asn1_time_parse_general (const gchar *time, gssize n_time)
 {
 	struct tm when;
 	gint offset;
 	
 	g_return_val_if_fail (time, -1);
 	
-	if (!parse_general_time (time, &when, &offset))
+	if (n_time == -1)
+		n_time = strlen (time);
+
+	if (!parse_general_time (time, n_time, &when, &offset))
 		return -1;
 	
 	return when_to_time (&when, offset);
@@ -715,31 +752,32 @@ read_asn1_time (ASN1_TYPE asn, const gchar *part, struct tm *when, gint *offset)
 	g_assert (when);
 	g_assert (offset);
 
-	len = sizeof (ttime) - 1;
+	len = sizeof (ttime);
 	res = asn1_read_value (asn, part, ttime, &len);
 	if (res != ASN1_SUCCESS)
 		return FALSE;
-		
+	--len; /* libtasn1 returns the null terminator in this count */
+
 	/* CHOICE */
-	if (strcmp (ttime, "generalTime") == 0) {
+	if (ascii_length_equals ("generalTime", ttime, len)) {
 		name = g_strconcat (part, ".generalTime", NULL);
 		len = sizeof (ttime) - 1;
 		res = asn1_read_value (asn, name, ttime, &len);
 		g_free (name);
 		if (res != ASN1_SUCCESS)
 			return FALSE;
-		return parse_general_time (ttime, when, offset);
+		return parse_general_time (ttime, len - 1, when, offset);
 		
 	/* UTCTIME */
-	} else {
+	} else if (ascii_length_equals ("utcTime", ttime, len)) {
 		name = g_strconcat (part, ".utcTime", NULL);
 		len = sizeof (ttime) - 1;
 		res = asn1_read_value (asn, name, ttime, &len);
 		g_free (name);
 		if (res != ASN1_SUCCESS)
 			return FALSE;
-		return parse_utc_time (ttime, when, offset);
-    	}
+		return parse_utc_time (ttime, len - 1, when, offset);
+	}
 
 	return FALSE;	
 }
@@ -777,407 +815,3 @@ egg_asn1_read_date (ASN1_TYPE asn, const gchar *part, GDate *date)
 	g_date_set_dmy (date, when.tm_mday, when.tm_mon + 1, when.tm_year + 1900);
 	return g_date_valid (date);
 }
-
-/* -------------------------------------------------------------------------------
- * Reading DN's
- */
-
-typedef struct _PrintableOid {
-	GQuark oid;
-	const gchar *oidstr;
-	const gchar *attr;
-	const gchar *description;
-	gboolean is_choice;
-} PrintableOid;
-
-static PrintableOid printable_oids[] = {
-	{ 0, "0.9.2342.19200300.100.1.25", "DC", N_("Domain Component"), FALSE },
-	{ 0, "0.9.2342.19200300.100.1.1", "UID", N_("User ID"), TRUE },
-
-	{ 0, "1.2.840.113549.1.9.1", "EMAIL", N_("Email"), FALSE },
-	{ 0, "1.2.840.113549.1.9.7", NULL, NULL, TRUE },
-	{ 0, "1.2.840.113549.1.9.20", NULL, NULL, FALSE },
-	
-	{ 0, "1.3.6.1.5.5.7.9.1", "dateOfBirth", N_("Date of Birth"), FALSE },
-	{ 0, "1.3.6.1.5.5.7.9.2", "placeOfBirth", N_("Place of Birth"), FALSE },
-	{ 0, "1.3.6.1.5.5.7.9.3", "gender", N_("Gender"), FALSE },
-        { 0, "1.3.6.1.5.5.7.9.4", "countryOfCitizenship", N_("Country of Citizenship"), FALSE },
-        { 0, "1.3.6.1.5.5.7.9.5", "countryOfResidence", N_("Country of Residence"), FALSE },
-
-	{ 0, "2.5.4.3", "CN", N_("Common Name"), TRUE },
-	{ 0, "2.5.4.4", "surName", N_("Surname"), TRUE },
-	{ 0, "2.5.4.5", "serialNumber", N_("Serial Number"), FALSE },
-	{ 0, "2.5.4.6", "C", N_("Country"), FALSE, },
-	{ 0, "2.5.4.7", "L", N_("Locality"), TRUE },
-	{ 0, "2.5.4.8", "ST", N_("State"), TRUE },
-	{ 0, "2.5.4.9", "STREET", N_("Street"), TRUE },
-	{ 0, "2.5.4.10", "O", N_("Organization"), TRUE },
-	{ 0, "2.5.4.11", "OU", N_("Organizational Unit"), TRUE },
-	{ 0, "2.5.4.12", "T", N_("Title"), TRUE },
-	{ 0, "2.5.4.20", "telephoneNumber", N_("Telephone Number"), FALSE },
-	{ 0, "2.5.4.42", "givenName", N_("Given Name"), TRUE },
-	{ 0, "2.5.4.43", "initials", N_("Initials"), TRUE },
-	{ 0, "2.5.4.44", "generationQualifier", N_("Generation Qualifier"), TRUE },
-	{ 0, "2.5.4.46", "dnQualifier", N_("DN Qualifier"), FALSE },
-	{ 0, "2.5.4.65", "pseudonym", N_("Pseudonym"), TRUE },
-
-	{ 0, NULL, NULL, NULL, FALSE }
-};
-
-static void
-init_printable_oids (void)
-{
-	static volatile gsize inited_oids = 0;
-	int i;
-	
-	if (g_once_init_enter (&inited_oids)) {
-		for (i = 0; printable_oids[i].oidstr != NULL; ++i)
-			printable_oids[i].oid = g_quark_from_static_string (printable_oids[i].oidstr);
-		g_once_init_leave (&inited_oids, 1);
-	}
-}
-
-static PrintableOid*
-dn_find_printable (GQuark oid)
-{
-	int i;
-	
-	g_return_val_if_fail (oid != 0, NULL);
-	
-	for (i = 0; printable_oids[i].oidstr != NULL; ++i) {
-		if (printable_oids[i].oid == oid)
-			return &printable_oids[i];
-	}
-	
-	return NULL;
-}
-
-static const char HEXC[] = "0123456789ABCDEF";
-
-static gchar*
-dn_print_hex_value (const guchar *data, gsize len)
-{
-	GString *result = g_string_sized_new (len * 2 + 1);
-	gsize i;
-	
-	g_string_append_c (result, '#');
-	for (i = 0; i < len; ++i) {
-		g_string_append_c (result, HEXC[data[i] >> 4 & 0xf]);
-		g_string_append_c (result, HEXC[data[i] & 0xf]);
-	}
-	
-	return g_string_free (result, FALSE);
-}
-
-static gchar* 
-dn_print_oid_value_parsed (PrintableOid *printable, const guchar *data, gsize len)
-{
-	const gchar *asn_name;
-	ASN1_TYPE asn1;
-	gchar *part;
-	gchar *value;
-	
-	g_assert (printable);
-	g_assert (data);
-	g_assert (len);
-	g_assert (printable->oid);
-	
-	asn_name = asn1_find_structure_from_oid (egg_asn1_get_pkix_asn1type (), 
-	                                         printable->oidstr);
-	g_return_val_if_fail (asn_name, NULL);
-	
-	part = g_strdup_printf ("PKIX1.%s", asn_name);
-	asn1 = egg_asn1_decode (part, data, len);
-	g_free (part);
-	
-	if (!asn1) {
-		g_message ("couldn't decode value for OID: %s", printable->oidstr);
-		return NULL;
-	}
-
-	value = (gchar*)egg_asn1_read_value (asn1, "", NULL, NULL);
-	
-	/*
-	 * If it's a choice element, then we have to read depending
-	 * on what's there.
-	 */
-	if (value && printable->is_choice) {
-		if (strcmp ("printableString", value) == 0 ||
-		    strcmp ("ia5String", value) == 0 ||
-		    strcmp ("utf8String", value) == 0 ||
-		    strcmp ("teletexString", value) == 0) {
-			part = value;
-			value = (gchar*)egg_asn1_read_value (asn1, part, NULL, NULL);
-			g_free (part);
-		} else {
-			g_free (value);
-			return NULL;
-		}
-	}
-
-	if (!value) {
-		g_message ("couldn't read value for OID: %s", printable->oidstr);
-		return NULL;
-	}
-
-	/* 
-	 * Now we make sure it's UTF-8. 
-	 */
-	if (!g_utf8_validate (value, -1, NULL)) {
-		gchar *hex = dn_print_hex_value ((guchar*)value, strlen (value));
-		g_free (value);
-		value = hex;
-	}
-	
-	return value;
-}
-
-static gchar*
-dn_print_oid_value (PrintableOid *printable, const guchar *data, gsize len)
-{
-	gchar *value;
-	
-	g_assert (data);
-	g_assert (len);
-	
-	if (printable) {
-		value = dn_print_oid_value_parsed (printable, data, len);
-		if (value != NULL)
-			return value;
-	}
-	
-	return dn_print_hex_value (data, len);
-}
-
-static gchar* 
-dn_parse_rdn (ASN1_TYPE asn, const gchar *part)
-{
-	PrintableOid *printable;
-	GQuark oid;
-	gchar *path;
-	guchar *value;
-	gsize n_value;
-	gchar *display;
-	gchar *result;
-	
-	g_assert (asn);
-	g_assert (part);
-	
-	path = g_strdup_printf ("%s.type", part);
-	oid = egg_asn1_read_oid (asn, path);
-	g_free (path);
-
-	if (!oid)
-		return NULL;
-	
-	path = g_strdup_printf ("%s.value", part);
-	value = egg_asn1_read_value (asn, path, &n_value, NULL);
-	g_free (path);
-
-	printable = dn_find_printable (oid);
-	
-	g_return_val_if_fail (value, NULL);
-	display = dn_print_oid_value (printable, value, n_value);
-	
-	result = g_strconcat (printable && printable->attr ? printable->attr : g_quark_to_string (oid), 
-			      "=", display, NULL);
-	g_free (display);
-	
-	return result;
-}
-
-gchar*
-egg_asn1_read_dn (ASN1_TYPE asn, const gchar *part)
-{
-	gboolean done = FALSE;
-	GString *result;
-	gchar *path;
-	gchar *rdn;
-	gint i, j;
-	
-	g_return_val_if_fail (asn, NULL);
-	g_return_val_if_fail (part, NULL);
-	
-	init_printable_oids ();
-	
-	result = g_string_sized_new (64);
-	
-	/* Each (possibly multi valued) RDN */
-	for (i = 1; !done; ++i) {
-		
-		/* Each type=value pair of an RDN */
-		for (j = 1; TRUE; ++j) {
-			path = g_strdup_printf ("%s%s?%u.?%u", part ? part : "", 
-			                        part ? "." : "", i, j);
-			rdn = dn_parse_rdn (asn, path);
-			g_free (path);
-
-			if (!rdn) {
-				done = j == 1;
-				break;
-			}
-			
-			/* Account for multi valued RDNs */
-			if (j > 1)
-				g_string_append (result, "+");
-			else if (i > 1)
-				g_string_append (result, ", ");
-			
-			g_string_append (result, rdn);
-			g_free (rdn);
-		}
-	}
-
-	/* Returns null when string is empty */
-	return g_string_free (result, (result->len == 0));
-}
-
-gchar*
-egg_asn1_read_dn_part (ASN1_TYPE asn, const gchar *part, const gchar *match)
-{
-	PrintableOid *printable = NULL;
-	gboolean done = FALSE;
-	guchar *value;
-	gsize n_value;
-	gchar *path;
-	GQuark oid;
-	gint i, j;
-	
-	g_return_val_if_fail (asn, NULL);
-	g_return_val_if_fail (part, NULL);
-	g_return_val_if_fail (match, NULL);
-	
-	init_printable_oids ();
-	
-	/* Each (possibly multi valued) RDN */
-	for (i = 1; !done; ++i) {
-		
-		/* Each type=value pair of an RDN */
-		for (j = 1; TRUE; ++j) {
-			path = g_strdup_printf ("%s%s?%u.?%u.type", 
-			                        part ? part : "", 
-			                        part ? "." : "", i, j);
-			oid = egg_asn1_read_oid (asn, path);
-			g_free (path);
-
-			if (!oid) {
-				done = j == 1;
-				break;
-			}
-			
-			/* Does it match either the OID or the displayable? */
-			if (g_ascii_strcasecmp (g_quark_to_string (oid), match) != 0) {
-				printable = dn_find_printable (oid);
-				if (!printable || !printable->attr || 
-				    !g_ascii_strcasecmp (printable->attr, match) == 0)
-					continue;
-			}
-
-			path = g_strdup_printf ("%s%s?%u.?%u.value", 
-			                        part ? part : "", 
-			                        part ? "." : "", i, j);
-			value = egg_asn1_read_value (asn, path, &n_value, NULL);
-			g_free (path);
-			
-			g_return_val_if_fail (value, NULL);
-			return dn_print_oid_value (printable, value, n_value);
-		}
-	}
-	
-	return NULL;
-}
-
-gboolean
-egg_asn1_dn_parse (ASN1_TYPE asn, const gchar *part, 
-                   EggAsn1DnCallback callback, gpointer user_data)
-{
-	gboolean done = FALSE;
-	gchar *path;
-	guchar *value;
-	gsize n_value;
-	GQuark oid;
-	guint i, j;
-	
-	g_return_val_if_fail (asn, FALSE);
-	
-	init_printable_oids ();
-	
-	/* Each (possibly multi valued) RDN */
-	for (i = 1; !done; ++i) {
-		
-		/* Each type=value pair of an RDN */
-		for (j = 1; TRUE; ++j) {
-			
-			/* Dig out the type */
-			path = g_strdup_printf ("%s%s?%u.?%u.type", 
-			                        part ? part : "", 
-			                        part ? "." : "", i, j);
-			oid = egg_asn1_read_oid (asn, path);
-			g_free (path);
-
-			if (!oid) {
-				done = j == 1;
-				break;
-			}
-
-			/* Print the value as nicely as we can */
-			path = g_strdup_printf ("%s%s?%u.?%u.value", 
-			                        part ? part : "", 
-			                        part ? "." : "", i, j);
-			value = egg_asn1_read_value (asn, path, &n_value, NULL);
-			g_free (path);
-
-			if (!value) {
-				done = j == 1;
-				break;
-			}
-			
-			if (callback) 
-				(callback) (i, oid, value, n_value, user_data);
-			
-			g_free (value);
-		}
-	}
-	
-	return i > 1;
-}
-
-const gchar*
-egg_asn1_dn_oid_attr (GQuark oid)
-{
-	PrintableOid *printable;
-	
-	g_return_val_if_fail (oid, NULL);
-	
-	printable = dn_find_printable (oid);
-	if (!printable)
-		return g_quark_to_string (oid);
-	
-	return printable->attr;	
-}
-
-const gchar*
-egg_asn1_dn_oid_desc (GQuark oid)
-{
-	PrintableOid *printable;
-	
-	g_return_val_if_fail (oid, NULL);
-	
-	printable = dn_find_printable (oid);
-	if (!printable)
-		return g_quark_to_string (oid);
-	
-	return gettext (printable->description);
-}
-
-gchar*
-egg_asn1_dn_print_value (GQuark oid, const guchar *value, gsize n_value)
-{
-	PrintableOid *printable;
-	
-	g_return_val_if_fail (oid, NULL);
-	g_return_val_if_fail (value || !n_value, NULL);
-	
-	printable = dn_find_printable (oid);
-	return dn_print_oid_value (printable, value, n_value);
-}
